@@ -7,7 +7,7 @@ use strict;
 use File::Find ();
 use urpm::msg;
 use urpm::download;
-use urpm::util;
+use urpm::util qw(basename begins_with cat_ cat_utf8 dirname file2absolute_file member);
 use urpm::sys;
 use urpm::cfg;
 use urpm::md5sum;
@@ -15,7 +15,7 @@ use urpm::md5sum;
 # perl_checker: require urpm::media
 # perl_checker: require urpm::parallel
 
-our $VERSION = '7.9';
+our $VERSION = '8.00';
 our @ISA = qw(URPM Exporter);
 our @EXPORT_OK = ('file_from_local_url', 'file_from_local_medium', 'is_local_medium');
 
@@ -52,7 +52,7 @@ sub xml_info_policies() { qw(never on-demand update-only always) }
 sub default_options {
     { 
 	'split-level' => 1,
-	'split-length' => 8,
+	'split-length' => 50,
 	'verify-rpm' => 1,
 	'post-clean' => 1,
 	'xml-info' => 'on-demand',
@@ -79,6 +79,52 @@ B<media>: [ {
    unknown_media_info => bool, 
  } ],
 
+B<options>: hashref of urpm options
+
+several paths:
+
+=over
+
+B<config>: path of  urpmi.cfg (/etc/urpmi/urpmi.cfg)
+
+B<mediacfgdir>: path of mediacfg.d (/etc/urpmi/mediacfg.d)
+
+B<skiplist>: path of skip.list (/etc/urpmi/skip.list),
+
+B<instlist>: path of inst.list (/etc/urpmi/inst.list),
+
+B<prefer_list>: path of prefer.list (/etc/urpmi/prefer.list),
+
+B<prefer_vendor_list>: path of prefer.vendor.list (/etc/urpmi/prefer.vendor.list),
+
+B<private_netrc>: path of netrc (/etc/urpmi/netrc),
+
+B<statedir>: state directory (/var/lib/urpmi),
+
+B<cachedir>: cache directory (/var/cache/urpmi),
+
+B<root>: path of the rooted system (when using global urpmi config),
+
+B<urpmi_root>: path of the rooted system (when both urpmi & rpmdb are chrooted)
+
+=back
+
+Several subs:
+
+=over
+
+B<fatal>: sub for relaying fatal errors (should popup in GUIes)
+
+B<error>: sub for relaying other errors
+
+B<log>: sub for relaying messages if --verbose
+
+B<print>: sub for always displayed messages, enable to redirect output for eg: installer
+
+B<info>: sub for messages displayed unless --quiet
+
+=back
+
 All C<URPM> methods are available on an urpm object.
 
 =cut
@@ -107,6 +153,12 @@ sub new {
     $self;
 }
 
+=item urpm->new_parse_cmdline()
+
+Like urpm->new but also parse the command line and parse the configuration file.
+
+=cut
+
 sub new_parse_cmdline {
     my ($class) = @_;
     my $urpm = $class->new;
@@ -550,7 +602,7 @@ __END__
 The L<URPM> package is used to manipulate at a lower level synthesis and rpm
 files.
 
-See also submodules: L<urpm::args>, L<urpm::bug_report>,
+See also submodules: L<gurpmi>, L<urpm::args>, L<urpm::bug_report>,
 L<urpm::cdrom>, L<urpm::cfg>, L<urpm::download>, L<urpm::get_pkgs>,
 L<urpm::install>, L<urpm::ldap>, L<urpm::lock>, L<urpm::main_loop>,
 L<urpm::md5sum>, L<urpm::media>, L<urpm::mirrors>, L<urpm::msg>,
@@ -565,7 +617,7 @@ Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005 MandrakeSoft SA
 
 Copyright (C) 2005-2010 Mandriva SA
 
-Copyright (C) 2011-2012 Mageia SA
+Copyright (C) 2011-2013 Mageia SA
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
