@@ -5,12 +5,12 @@ package urpm::install;
 use strict;
 use urpm;
 use urpm::msg;
-use urpm::util;
+use urpm::util qw(cat_utf8 member);
 
 
 =head1 NAME
 
-urpm::install - Package installation transactin routines for urpmi
+urpm::install - Package installation transaction routines for urpmi
 
 =head1 SYNOPSIS
 
@@ -88,26 +88,38 @@ sub transaction_set_to_string {
 
 Standard logger for transactions
 
+See L<URPM> for parameters
+
 =cut
 
 # install logger callback
+my ($erase_logger, $index, $total_pkg, $uninst_count);
 sub install_logger {
-    my ($urpm, $type, $id, $subtype, $amount, $total) = @_;
-    my $pkg = defined $id && $urpm->{depslist}[$id];
-    my $total_pkg = $urpm->{nb_install};
+    my ($urpm, $type, undef, $subtype, $amount, $total) = @_;
     local $| = 1;
 
     if ($subtype eq 'start') {
 	$urpm->{logger_progress} = 0;
 	if ($type eq 'trans') {
+	    $total_pkg = $urpm->{nb_install};
 	    $urpm->{logger_count} ||= 0;
+	    $uninst_count = 0;
 	    my $p = N("Preparing...");
 	    print $p, " " x (33 - length $p);
 	} else {
-	    my $pname = $pkg ? $pkg->name : '';
-	    ++$urpm->{logger_count} if $pname;
-	    my $cnt = $pname ? $urpm->{logger_count} : '-';
-	    $pname ||= N("[repackaging]");
+	    my $pname;
+	    my $cnt;
+	    if ($type eq 'uninst') {
+		$total_pkg = $urpm->{trans}->NElements - $index if !$uninst_count;
+		$cnt = ++$uninst_count;
+		$pname = N("removing %s", $urpm->{trans}->Element_fullname($index));
+		$erase_logger->($urpm, undef, undef, $subtype);
+	    } else {
+		$pname = $urpm->{trans}->Element_name($index);
+		++$urpm->{logger_count} if $pname;
+		$cnt = $pname ? $urpm->{logger_count} : '-';
+	    }
+	    $index++;
 	    my $s = sprintf("%9s: %-22s", $cnt . "/" . $total_pkg, $pname);
 	    print $s;
 	    $s =~ / $/ or printf "\n%9s  %-22s", '', '';
@@ -149,6 +161,8 @@ sub get_README_files {
 		foreach my $i (0 .. $trans->NElements - 1) {
 		    $trans->Element_name($i) eq $pkg->name or next;
 
+		    # handle README.<version>-<release>.upgrade.urpmi:
+		    # the content is displayed when upgrading from rpm older than <version>
 		    my $vr = $trans->Element_version($i) . '-' . $trans->Element_release($i);
 		    if (URPM::ranges_overlap("== $vr", "< $version")) {
 			$valid = 1;
@@ -175,39 +189,8 @@ sub options {
     );
 }
 
-=item install($urpm, $remove, $install, $upgrade, %options)
-
-Install packages according to each hash (remove, install or upgrade).
-
-options: 
-     test, excludepath, nodeps, noorder (unused), delta, 
-     callback_inst, callback_trans, callback_report_uninst,
-     post_clean_cache, verbose
-  (more options for trans->run)
-     excludedocs, nosize, noscripts, oldpackage, replacepkgs, justdb, ignorearch
-
-=cut
-
-sub install {
-    my ($urpm, $remove, $install, $upgrade, %options) = @_;
-    $options{translate_message} = 1;
-
-    my $db = urpm::db_open_or_die_($urpm, !$options{test}); #- open in read/write mode unless testing installation.
-
-    my $trans = $db->create_transaction;
-    if ($trans) {
-	sys_log("transaction on %s (remove=%d, install=%d, upgrade=%d)", $urpm->{root} || '/', scalar(@{$remove || []}), scalar(values %$install), scalar(values %$upgrade));
-	$urpm->{log}(N("created transaction for installing on %s (remove=%d, install=%d, upgrade=%d)", $urpm->{root} || '/',
-		       scalar(@{$remove || []}), scalar(values %$install), scalar(values %$upgrade)));
-    } else {
-	return N("unable to create transaction");
-    }
-
-    $trans->set_script_fd($options{script_fd}) if $options{script_fd};
-
-    my ($update, @errors) = 0;
-    my @produced_deltas;
-
+sub _schedule_packages_for_erasing {
+    my ($urpm, $trans, $remove) = @_;
     foreach (@$remove) {
 	if ($trans->remove($_)) {
 	    $urpm->{debug} and $urpm->{debug}("trans: scheduling removal of $_");
@@ -215,22 +198,40 @@ sub install {
 	    $urpm->{error}("unable to remove package " . $_);
 	}
     }
-    my @trans_pkgs;
+}
+
+sub _apply_delta_rpm {
+    my ($urpm, $path, $mode, $pkg) = @_;
+    my $true_rpm = urpm::sys::apply_delta_rpm($path, "$urpm->{cachedir}/rpms", $pkg);
+    my $true_pkg;
+    if ($true_rpm) {
+	if (my ($id) = $urpm->parse_rpm($true_rpm)) {
+	    $true_pkg = defined $id && $urpm->{depslist}[$id];
+	    $mode->{$id} = $true_rpm;
+	} else {
+	    $urpm->{error}("Failed to parse $true_pkg");
+	}
+    } else {
+	$urpm->{error}(N("unable to extract rpm from delta-rpm package %s", $path));
+    }
+    $true_rpm, $true_pkg;
+}
+
+sub _schedule_packages {
+    my ($urpm, $trans, $install, $upgrade, %options) = @_;
+    my $update = 0;
+    my (@trans_pkgs, @produced_deltas);
     foreach my $mode ($install, $upgrade) {
 	foreach (keys %$mode) {
 	    my $pkg = $urpm->{depslist}[$_];
 	    $pkg->update_header($mode->{$_}, keep_all_tags => 1);
+	    my ($true_rpm, $true_pkg);
 	    if ($pkg->payload_format eq 'drpm') { #- handle deltarpms
-		my $true_rpm = urpm::sys::apply_delta_rpm($mode->{$_}, "$urpm->{cachedir}/rpms", $pkg);
-		if ($true_rpm) {
-		    push @produced_deltas, ($mode->{$_} = $true_rpm); #- fix path
-		} else {
-		    $urpm->{error}(N("unable to extract rpm from delta-rpm package %s", $mode->{$_}));
-		}
+		($true_rpm, $true_pkg) = _apply_delta_rpm($urpm, $mode->{$_}, $mode, $pkg);
+		push @produced_deltas, ($mode->{$_} = $true_rpm); #- fix path
 	    }
-	    if ($trans->add($pkg, update => $update,
-		    $options{excludepath} ? (excludepath => [ split /,/, $options{excludepath} ]) : ()
-	    )) {
+	    if ($trans->add($true_pkg || $pkg, update => $update,
+		    $options{excludepath} ? (excludepath => [ split /,/, $options{excludepath} ]) : ())) {
 		$urpm->{debug} and $urpm->{debug}(
 		    sprintf('trans: scheduling %s of %s (id=%d, file=%s)', 
 		      $update ? 'update' : 'install', 
@@ -248,57 +249,120 @@ sub install {
 	}
 	++$update;
     }
+    \@produced_deltas, @trans_pkgs;
+}
+
+sub _get_callbacks {
+    my ($urpm, $db, $trans, $options, $install, $upgrade, $have_pkgs) = @_;
+    $index = 0;
+    my $fh;
+
+    my $is_test = $options->{test}; # fix circular reference
+    #- assume default value for some parameter.
+    $options->{delta} ||= 1000;
+
+    #- ensure perl does not create a circular reference below, otherwise all this won't be collected,
+    #  and rpmdb won't be closed:
+    my ($callback_open_helper, $callback_close_helper) = ($options->{callback_open_helper}, $options->{callback_close_helper});
+    $options->{callback_open} = sub {
+	my ($_data, $_type, $id) = @_;
+	$callback_open_helper and $callback_open_helper->(@_);
+	$fh = urpm::sys::open_safe($urpm, '<', $install->{$id} || $upgrade->{$id});
+	$fh ? fileno $fh : undef;
+    };
+    $options->{callback_close} = sub {
+	my ($urpm, undef, $pkgid) = @_;
+	return unless defined $pkgid;
+	$callback_close_helper and $callback_close_helper->($db, @_);
+	get_README_files($urpm, $trans, $urpm->{depslist}[$pkgid]) if !$is_test;
+	close $fh if defined $fh;
+    };
+
+    #- ensure perl does not create a circular reference below, otherwise all this won't be collected,
+    #  and rpmdb won't be closed
+    my $verbose = $options->{verbose};
+    $erase_logger = sub {
+	my ($urpm, undef, undef, $subtype) = @_;
+
+	if ($subtype eq 'start') {
+	    my ($name, $fullname) = ($trans->Element_name($index), $trans->Element_fullname($index));
+	    my @previous = map { $trans->Element_name($_) } 0 .. ($index - 1);
+	    # looking at previous packages in transaction
+	    # we should be looking only at installed packages, but it should not give a different result
+	    if (member($name, @previous)) {
+		$urpm->{log}("removing upgraded package $fullname");
+	    } else {
+		$urpm->{print}(N("removing package %s", $fullname)) if $verbose >= 0;
+	    }
+	}
+    };
+
+    $options->{callback_uninst} ||= $options->{verbose} >= 0 ? \&install_logger : $erase_logger;
+
+    $options->{callback_error} ||= sub {
+	my ($urpm, undef, $id, $subtype) = @_;
+	my $n = $urpm->{depslist}[$id]->fullname;
+	$urpm->{error}("ERROR: '$subtype' failed for $n: ");
+    };
+
+    if ($options->{verbose} >= 0 && $have_pkgs) {
+	$options->{callback_inst}  ||= \&install_logger;
+	$options->{callback_trans} ||= \&install_logger;
+    }
+}
+
+=item install($urpm, $remove, $install, $upgrade, %options)
+
+Install packages according to each hash (remove, install or upgrade).
+
+options: 
+     test, excludepath, nodeps, noorder (unused), delta, 
+     callback_inst, callback_trans, callback_uninst,
+     callback_open_helper, callback_close_helper,
+     post_clean_cache, verbose
+  (more options for trans->run)
+     excludedocs, nosize, noscripts, oldpackage, replacepkgs, justdb, ignorearch
+
+See L<URPM> for callback parameters
+
+=cut
+
+#- side-effects: uses a $urpm->{readmes}
+sub install {
+    my ($urpm, $remove, $install, $upgrade, %options) = @_;
+    $options{translate_message} = 1;
+
+    my $db = urpm::db_open_or_die_($urpm, !$options{test}); #- open in read/write mode unless testing installation.
+
+    my $trans = $db->create_transaction;
+    if ($trans) {
+	my ($rm_count, $inst_count, $up_count) = (scalar(@{$remove || []}), scalar(values %$install), scalar(values %$upgrade));
+	sys_log("transaction on %s (remove=%d, install=%d, upgrade=%d)", $urpm->{root} || '/', $rm_count, $inst_count, $up_count);
+	$urpm->{log}(N("created transaction for installing on %s (remove=%d, install=%d, upgrade=%d)", $urpm->{root} || '/',
+		       $rm_count, $inst_count, $up_count));
+    } else {
+	return N("unable to create transaction");
+    }
+
+    $trans->set_script_fd($options{script_fd}) if $options{script_fd};
+
+    my @errors;
+
+    _schedule_packages_for_erasing($urpm, $trans, $remove);
+
+    my ($produced_deltas, @trans_pkgs) = _schedule_packages($urpm, $trans, $install, $upgrade, %options);
+
     if (!$options{nodeps} && (@errors = $trans->check(%options))) {
-    } elsif (!$options{noorder} && (@errors = $trans->order)) {
+    } elsif (!$options{noorder} && (@errors = $trans->order(%options))) {
     } else {
 	$urpm->{readmes} = {};
-	my $index;
-	my $fh;
-	my $is_test = $options{test}; # fix circular reference
-	#- assume default value for some parameter.
-	$options{delta} ||= 1000;
 
-	#- ensure perl does not create a circular reference below, otherwise all this won't be collected,
-	#  and rpmdb won't be closed:
-	my ($callback_open_helper, $callback_close_helper) = ($options{callback_open_helper}, $options{callback_close_helper});
-	$options{callback_open} = sub {
-	    my ($_data, $_type, $id) = @_;
-	    $index++;
-	    $callback_open_helper and $callback_open_helper->(@_);
-	    $fh = urpm::sys::open_safe($urpm, '<', $install->{$id} || $upgrade->{$id});
-	    $fh ? fileno $fh : undef;
-	};
-	$options{callback_close} = sub {
-	    my ($urpm, undef, $pkgid) = @_;
-	    return unless defined $pkgid;
-	    $callback_close_helper and $callback_close_helper->($db, @_);
-	    get_README_files($urpm, $trans, $urpm->{depslist}[$pkgid]) if !$is_test;
-	    close $fh if defined $fh;
-	};
-	#- ensure perl does not create a circular reference below, otherwise all this won't be collected,
-	#  and rpmdb won't be closed
-	my ($verbose, $callback_report_uninst) = ($options{verbose}, $options{callback_report_uninst});
-	$options{callback_uninst} = sub { 	    
-	    my ($_urpm, undef, undef, $subtype) = @_;
-	    if ($subtype eq 'start') {
-		my ($name, $fullname) = ($trans->Element_name($index), $trans->Element_fullname($index));
-		my @previous = map { $trans->Element_name($_) } 0 .. ($index - 1);
-		# looking at previous packages in transaction
-		# we should be looking only at installed packages, but it should not give a different result
-		if (member($name, @previous)) {
-		    $urpm->{log}("removing upgraded package $fullname");
-		} else {
-		    $callback_report_uninst and $callback_report_uninst->(N("Removing package %s", $fullname));
-		    $urpm->{print}(N("removing package %s", $fullname)) if $verbose >= 0;
-		}
-		$index++;
-	    }
-	};
-	if ($options{verbose} >= 0 && @trans_pkgs) {
-	    $options{callback_inst}  ||= \&install_logger;
-	    $options{callback_trans} ||= \&install_logger;
-	}
+	_get_callbacks($urpm, $db, $trans, \%options, $install, $upgrade, scalar @trans_pkgs);
+
+	local $urpm->{trans} = $trans;
 	@errors = $trans->run($urpm, %options);
+	delete $urpm->{trans};
+	undef $erase_logger;
 
 	#- don't clear cache if transaction failed. We might want to retry.
 	if (!@errors && !$options{test} && $options{post_clean_cache}) {
@@ -311,7 +375,7 @@ sub install {
 	    }
 	}
 
-	if ($options{verbose} >= 0) {
+	if ($options{verbose} >= 0 && !$options{justdb}) {
 	    foreach (keys %{$urpm->{readmes}}) {
 		$urpm->{print}("-" x 70 .  "\n" .
                                  N("More information on package %s", $urpm->{readmes}{$_}));
@@ -320,7 +384,8 @@ sub install {
 	    }
 	}
     }
-    unlink @produced_deltas;
+
+    unlink @$produced_deltas;
 
     urpm::sys::may_clean_rpmdb_shared_regions($urpm, $options{test});
 
@@ -341,6 +406,6 @@ Copyright (C) 1999-2005 MandrakeSoft SA
 
 Copyright (C) 2005-2010 Mandriva SA
 
-Copyright (C) 2011-2012 Mageia
+Copyright (C) 2011-2013 Mageia
 
 =cut
