@@ -3,9 +3,9 @@ package urpm::media;
 # $Id: media.pm 270394 2010-07-30 00:48:07Z misc $
 
 use strict;
-use urpm 'file_from_local_medium', 'is_local_medium';
+use urpm qw(file_from_local_medium is_local_medium);
 use urpm::msg;
-use urpm::util;
+use urpm::util qw(any append_to_file basename cat_ difference2 dirname intersection member output_safe begins_with copy_and_own file_size offset_pathname reduce_pathname);
 use urpm::removable;
 use urpm::lock;
 use urpm::md5sum;
@@ -731,12 +731,12 @@ sub configure {
 	}
 	_auto_update_media($urpm, %options);
 
-	_pick_mirror_if_needed($urpm, $_, '') foreach non_ignored_media($urpm, $options{update});
+	_pick_mirror_if_needed($urpm, $_, '') foreach non_ignored_media($urpm);
 
-	parse_media($urpm, \%options) if !$options{nodepslist};
-
-    #- determine package to withdraw (from skip.list file) only if something should be withdrawn.
     if (!$options{nodepslist}) {
+	parse_media($urpm, \%options);
+
+	#- determine package to withdraw (from skip.list file) only if something should be withdrawn.
 	_compute_flags_for_skiplist($urpm, $options{cmdline_skiplist}) if !$options{no_skiplist};
 	_compute_flags_for_instlist($urpm);
     }
@@ -750,11 +750,59 @@ sub _auto_update_media {
     $options{callback} = delete $options{download_callback};
 
     foreach (grep { _is_remote_virtual($_) || $urpm->{options}{'auto-update'} } 
-	       non_ignored_media($urpm, $options{update})) {
+	       non_ignored_media($urpm)) {
 	_update_medium($urpm, $_, %options);
     }
 }
 
+
+=item needed_extra_media($urpm)
+
+Return 2 booleans telling whether nonfree & tainted packages are installed respectively.
+
+=cut
+
+sub needed_extra_media {
+    my ($urpm) = @_;
+    my $db = urpm::db_open_or_die_($urpm);
+    my ($nonfree, $tainted);
+    $db->traverse(sub {
+	my ($pkg) = @_;
+	return if $nonfree && $tainted;
+	my $rel = $pkg->release;
+	$nonfree ||= $rel =~ /nonfree$/;
+	$tainted ||= $rel =~ /tainted$/;
+    });
+    ($nonfree, $tainted);
+}
+
+sub is_media_to_add_by_default {
+    my ($urpm, $distribconf, $medium, $product_id, $nonfree, $tainted) = @_;
+    my $add_by_default = !$distribconf->getvalue($medium, 'noauto');
+    my @media_types = split(':', $distribconf->getvalue($medium, 'media_type'));
+    return $add_by_default if !@media_types;
+    if ($product_id->{product} eq 'Free') {
+	if (member('non-free', @media_types)) {
+	    $urpm->{log}(N("ignoring non-free medium `%s'", $medium));
+	    $add_by_default = 0;
+	}
+    } else {
+	my $non_regular_medium = intersection(\@media_types, [ qw(backports debug source testing) ]);
+	if (!$add_by_default && !$non_regular_medium) {
+	    my $medium_name = $distribconf->getvalue($medium, 'name') || '';
+	    if ($medium_name =~ /Nonfree/ && $nonfree) {
+		$add_by_default = 1;
+		$urpm->{log}(N("un-ignoring non-free medium `%s' b/c nonfree packages are installed", $medium_name));
+	    }
+	    if ($medium_name =~ /Tainted/ && $tainted) {
+		$add_by_default = 1;
+		$urpm->{log}(N("un-ignoring tainted medium `%s' b/c tainted packages are installed", $medium_name));
+	    }
+	}
+    }
+    $add_by_default;
+}
+
 sub non_ignored_media {
     my ($urpm, $b_only_marked_update) = @_;
 
@@ -764,16 +812,13 @@ sub non_ignored_media {
 sub all_media_to_update {
     my ($urpm, $b_only_marked_update) = @_;
 
-    grep { !$_->{ignore}
-	     && !$_->{static} && !urpm::is_cdrom_url($_->{url}) && !$_->{iso}
-	     && (!$b_only_marked_update || $_->{update});
-	} @{$urpm->{media} || []};
+    grep {  !$_->{static} && !urpm::is_cdrom_url($_->{url}) && !$_->{iso} } non_ignored_media($urpm, $b_only_marked_update);
 }
 
 sub parse_media {
     my ($urpm, $options) = @_;
 
-    foreach (non_ignored_media($urpm, $options->{update})) {
+    foreach (non_ignored_media($urpm)) {
 	delete @$_{qw(start end)};
 	_parse_synthesis_or_ignore($urpm, $_, $options->{callback});
 
@@ -1094,6 +1139,7 @@ sub add_distrib_media {
 
     require urpm::mirrors;
     my $product_id = urpm::mirrors::parse_LDAP_namespace_structure(cat_('/etc/product.id'));
+    my ($nonfree, $tainted) = needed_extra_media($urpm);
 
     foreach my $media ($distribconf->listmedia) {
         my $media_name = $distribconf->getvalue($media, 'name') || '';
@@ -1111,14 +1157,8 @@ sub add_distrib_media {
 	    $is_update_media or next;
 	}
 
-        my $add_by_default = !$distribconf->getvalue($media, 'noauto');
-        my @media_types = split(':', $distribconf->getvalue($media, 'media_type'));
-        if ($product_id->{product} eq 'Free') {
-            if (member('non-free', @media_types)) {
-                $urpm->{log}(N("ignoring non-free medium `%s'", $media));
-                $add_by_default = 0;
-            }
-        }
+        my $add_by_default = is_media_to_add_by_default($urpm, $distribconf, $media, $product_id, $nonfree, $tainted);
+
 	my $ignore;
         if ($options{ask_media}) {
             $options{ask_media}->($media_name, $add_by_default) or next;
@@ -2028,7 +2068,7 @@ sub update_media {
 	$_->{modified} ||= 1 foreach all_media_to_update($urpm);
     }
 
-    update_those_media($urpm, [ grep { $_->{modified} } non_ignored_media($urpm, $options{update}) ], %options);
+    update_those_media($urpm, [ grep { $_->{modified} } non_ignored_media($urpm) ], %options);
 }
 
 sub update_those_media {
