@@ -18,6 +18,7 @@ our @EXPORT = qw(add2hash_
     file2absolute_file
     file_size
     find
+    formatList
     intersection
     max
     member 
@@ -37,8 +38,6 @@ our @EXPORT = qw(add2hash_
     wc_l
 );
 
-(our $VERSION) = q($Revision: 271299 $) =~ /(\d+)/;
-
 sub min  { my $n = shift; $_ < $n and $n = $_ foreach @_; $n }
 sub max  { my $n = shift; $_ > $n and $n = $_ foreach @_; $n }
 
@@ -225,6 +224,8 @@ __END__
 
 urpm::util - Misc. utilities subs for urpmi
 
+Mostly a subset of L<MDK::Common>
+
 =head1 SYNOPSIS
 
 =head1 DESCRIPTION
